/*
 * Glewlwyd specific css file
 * Copyright 2019-2021 Nicolas Mora <mail@babelouest.org>
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 */
.btn-icon {
  margin-right: 5px;
}
.btn-icon-right {
  margin-left: 5px;
}
.error-input {
  width: 100%;
  margin-top: .25rem;
  font-size: 80%;
  color: #dc3545;
}
.img-thumb {
  width: 100%;
  max-width: 50px;
  height: 100%;
  max-height: 50px;
}
.img-medium {
  width: 100%;
  max-width: 100px;
  height: 100%;
  max-height: 100px;
}
.input-hidden {
  position: absolute;
  left: -9999px;
}
.toast-container {
  position: absolute;
  border-width: 1px;
  border-style: solid;
  border-radius: 5px;
  background: white;
  z-index: 9999;
}
.perfect-centering {
  position: absolute;
  top: 50%;
  left:50%;
  transform: translate(-50%,-50%);
}
.on-top {
  z-index: 9999;
}
.v-center {
  display: flex;
  align-items: center;
}
.required-field::before {
  margin-right: 5px;
  content: "*";
  color: red;
}
.logo-img {
  max-width: 40px;
  max-height: 40px;
}
.glwd-container {
  position: relative;
  min-height: 200px;
}
.glwd-card {
  margin-top: 20px;
  margin-bottom: 20px;
}
.glwd-select-user-picture {
  width: 30px;
  max-height: 21px;
  object-fit: scale-down;
}
.glwd-select-user-picto {
  font-size: 24px;
}
.glwd-select-user-label {
  width: 100%;
  margin-bottom: 0;
  padding-left: 0; 
  white-space: nowrap;
  overflow: hidden;
  text-overflow: ellipsis;
}
.glwd-select-user-col {
  display: inline;
}
.glwd-select-user-username {
  display: inline;
  padding-left: 0;
  white-space: nowrap;
  overflow: hidden;
  text-overflow: ellipsis;
}
.v-align-middle {
  vertical-align: middle;
}
.glwd-select-user-hidden-button {
  visibility: hidden;
  padding-left: 0;
  padding-right: 0
}
.glwd-no-padding-right {
  padding-right: 0;
}
.glwd-no-padding-left {
  padding-left: 0;
}
.glwd-hr-no-border {
  border: 0px;
}
.glwd-profile-user-picto-large {
  text-align: center;
  font-size: 48px;
}
.glwd-nav-picture-div {
  white-space: nowrap;
  overflow: hidden;
  text-overflow: ellipsis;
}
.glwd-nav-picture-image {
  max-width: 30px; 
  max-height: 21px;
  object-fit: scale-down;
  margin-right: 5px;
}
.glwd-nav-picture-spacer {
  max-width: 1px; 
}
.glwd-nav-picture-container {
  display: block;
  float: left;
}
.glwd-group-card {
  width: 18rem;
}

.glwd-nav-user-unavailable, .glwd-nav-user-unavailable:hover, .glwd-nav-user-unavailable:focus {
  cursor: default;
  color: #dcdcdc;
  pointer-events: none;
}

.glwd-toast {
  opacity: 0.9; important!
}

.upload {
  display:none;
}
